.setcpu "65816"
.smart
.feature force_range

;Included Files

.include "Header.asm"
.include "Macros.asm"
.include "2input.asm"
.include "ProcessOam.asm"
.include "Objects.asm"
.include "LoadGraphics.asm"
.include "Metasprite.asm"
;.include "AnimationEngine.asm"
;.include "VramFinder.asm"
;.include "ObjectTileUploader.asm"
;.include "PaletteChecker.asm"
;.include "PaletteUploader.asm"

;Define Stuff

ObjectTableSize = 6144

;Object Numbers
Object1 = 2
Object2 = 4

;Object Slot Registers
Identity = 0
XPosition = 2
YPosition = 4
XVelocity = 6
YVelocity = 8
HitboxWidth = 10
HitboxHeight = 12
FlipBits = 14
MetaspriteTableOffset = 16
AnimationFrame = 18
AnimationCounter = 20

;Button Numbers
RButton = 16
LButton = 32
XButton = 64
AButton = 128

RightDPad = 1
LeftDPad = 2
DownDPad = 4
UpDPad = 8
StartButton = 16
SelectButton = 32
YButton = 64
BButton = 128

WordBitRightDPad = 256
WordLeftDPad = 512
WordDownDPad = 1024
WordUpDPad = 2048
WordStartButton = 4096
WordSelectButton = 8192
WordYButton = 16384
WordBButton = 32768

;
; Game variables, located within 6502 zero page ($00-ff)
; 65816 has direct page which can range from $0000-ffff in region, but ca65
; only seems to understand "zp" being up to 256 bytes in size.
;
; See lorom256k.cfg and lda/tcd code in _reset
;
.segment "ZEROPAGE"

Joy1Data:                  .res 2
Joy2Data:                  .res 2
Joy1Press:                 .res 2
Joy2Press:                 .res 2
MapX:                      .res 2
MapY:                      .res 2
HFlipMask:                 .res 2
VFlipMask:                 .res 2
SpriteCount:               .res 2
MetaspriteCount:           .res 2
MetaspriteAttributes:      .res 2
NewObjectRequest:          .res 2
ObjectOffset:              .res 2
;NewPaletteRequest:         .res 2
;DMAPaletteRequestCounter:  .res 1
;DMAPaletteOffset:          .res 2

;
; SNES RAM area, $0200-1fff
; 
; See lorom256k.cfg
;

.segment "BSS"
SpriteBuf1:    .res $200   ; Sprite buffer for OAM DMA, 512 ($200) bytes
SpriteBuf1Size = *-SpriteBuf1
SpriteBuf2:    .res $20    ; Sprite buffer for OAM DMA, 32  ($20) bytes
SpriteBuf2Size = *-SpriteBuf2
SpriteBuf3:    .res $200   ; Sprite buffer for OAM DMA, 512  ($200) bytes
SpriteBuf3Size = *-SpriteBuf3
SpriteBufSize  = SpriteBuf1Size+SpriteBuf2Size
ObjectTable:    .res $1800
;TotalPalettesTable:    .res $20
;TotalPalettesTable2:    .res $20
;OnscreenPaletteTable:    .res $10

.segment "BSS7E"


; Assemble-time assertion; SNES DMA transfer can only handle 65536 bytes.
.assert (SpriteBufSize < $10000), error, "SpriteBufs area too big for SNES DMA transfer"

;
; Program/code area
;
.segment "CODE"

.proc SpriteInit
  php	
  rep #$30

  lda #$01
  ldx #0
: sta SpriteBuf3+1,x
  inx
  inx
  inx
  inx
  cpx #$0200
  bne :-

  plp
  rts
.endproc


.proc Main
  ;
  ; Initialize SNES, which includes setting up CPU registers properly,
  ; zeroing out RAM contents, setting up SNES registers per official
  ; Nintendo documentation, and many other things.
  ;
  InitializeSNES

  ; Make sure registers are proper size before calling these macros!
  ; We could add this to each macro, but that's up to you.
  rep #$30   ; A=16, X/Y=16
  sep #$20   ; A=8
  
  ; Load palette to make our pictures look correct
  LoadPalette BGPalette,     0, 96
  LoadPalette Player1Palette,     128, 16
  LoadPalette BulletPalette,     144, 16

  ; Load Tile and Character data to VRAM
  LoadBlockToVRAM Player1Tiles,		$0000, $0040, $0000
  LoadBlockToVRAM Player1Tiles,		$0100, $0040, $0040
  LoadBlockToVRAM Bullet1Tiles,		$0020, $0040, $0000
  LoadBlockToVRAM Bullet1Tiles,		$0120, $0040, $0040
  LoadBlockToVRAM BackgroundPics,	$2000, $A000, $0000	; 384 tiles * (8bit color)= 0x6000 bytes
  LoadBlockToVRAM BackgroundMap,	$7000, $2000, $0000	; 64x64 tiles = 4096 words = 8192 bytes

  jsr SpriteInit
  jsr JoyInit      ; setup joypads

  ; setup our walking sprite

;  rep #$30                  ; A=16, X/Y=16
;  lda #Object1
;  sta NewObjectRequest
;  jsr start_object_spawner
;
;  lda #Player1MetaspriteTable
;  sta ObjectTable+MetaspriteTableOffset,x



  jsr SetupVideo   ; setup the video modes and such, then turn on the screen

  rep #$30   ; A=16, X/Y=16
  sep #$20   ; A=8

  ; Enable NMI (when interrupts get re-enabled) and joypad reading
  lda #$81
  sta $4200

  cli              ; Restore interrupts -- we're all ready to go!

InfiniteLoop:
  jsr start_object_identifier
  wai                         ; Wait for interrupts to finish (NMI/VBlank)
  jmp InfiniteLoop            ; Do this forever

.endproc

;=========================================================================================
; VBlank
;=========================================================================================

.proc VBlank
  rep #$30   ; A=16, X/Y=16
  pha
  phx
  phy
  phd
  phb
  php
  lda #$0000
  tcd

  ; *********transfer sprite data

  sep #$20   ; A=8
  stz $2102                    ; set OAM address to 0
  stz $2103                    ; ...and no priority rotation

  stz $4300                    ; CPU -> PPU, auto-increment, write once
  lda #$04
  sta $4301                    ; DMA will write to register $2104 (OAM data write)
  ldy #.LOWORD(SpriteBuf1)
  sty $4302                    ; source offset of SpriteBuf1
  lda #.BANKBYTE(SpriteBuf1)
  sta $4304                    ; bank address of SpriteBuf1
  ldy #SpriteBufSize
  sty $4305                    ; transfer SpriteBufSize bytes (pre-calculated number)
  lda #$01
  sta $420b                    ; start DMA transfer

  ; update the map co-ordinates
  lda MapX
  sta $210d
  lda MapX+1
  sta $210d

  lda MapY
  sta $210e
  lda MapY+1
  sta $210e

  jsr GetInput    ; update joypad data

  lda $4210       ; clear NMI Flag

  plp
  plb
  pld
  ply
  plx
  pla
  rti
.endproc

;=========================================================================================
; SetupVideo
;=========================================================================================

.proc SetupVideo
  php
  rep #$30   ; A=16, X/Y=16
  sep #$20   ; A=8

  stz $211b
  lda #$01
  sta $211b
  stz $211c
  stz $211c
  stz $211d
  stz $211d
  stz $211e
  sta $211e
  lda #$80
  sta $211f
  stz $211f
  sta $2120
  stz $2120

  sta $2121
  lda #%01100000		;Sprites 32x32 or 64x64, character data at $0000 (word address)
  sta $2101         

  lda #$01		;Set video mode 1, 16x16 tiles (16 color BG1, 16 color BG2, 4 color BG3)
  sta $2105         

  lda #$71		;Set BG1's Tile Map VRAM offset to $7800 (word address)
  sta $2107		;   and the Tile Map size to 32 tiles x 32 tiles

  lda #$02		;Set BG1's Character VRAM offset to $4000 (word address)
  sta $210b

  lda #$11		;Turn on BG1 and BG2 and Sprites
  sta $212c

  lda #$ff		;Scroll BG2 down 1 pixel
  sta $2110
  sta $2110

  lda #$0f		;Turn on screen, full brightness
  sta $2100

  plp
  rts
.endproc

;=========================================================================================
; Segment for our binary data
;=========================================================================================
.segment "RODATA"
;=========================================================================================

Player1MetaspriteTable:
  .word $0001   ; Number of metasprite table entries below
        ;XPos  YPos  Size  Palette/Character
  .word $0000,$0000,$0000,$0000

Bullet1MetaspriteTable:
  .word $0001   ; Number of metasprite table entries below
        ;XPos  YPos  Size  Palette/Character
  .word $0000,$0000,$0000,$0202



AnimationTableBegining:

Player1Animation1Table:
  .word Player1Animation1Frame1, $0000

Bullet1Animation1Table:
  .word Bullet1Animation1Frame1, $0000

Player1Animation1Frame1:
  .word $0040, ^Player1Tiles, Player1Tiles, $0000

Bullet1Animation1Frame1:
  .word $0040, ^Bullet1Tiles, Bullet1Tiles, $0000

;=========================================================================================

; Map data
BackgroundMap:
  .incbin "GamePictures\hovertransport.map"

;=========================================================================================
; Color data
;=========================================================================================

BGPalette:
  .incbin "GamePictures\hovertransport.clr"

Player1Palette:
  .incbin "GamePictures\object1.clr"

BulletPalette:
  .incbin "GamePictures\object2.clr"

;=========================================================================================
; Tile data
;=========================================================================================

BackgroundPics:
  .incbin "GamePictures\hovertransport.pic"

Player1Tiles:
  .incbin "GamePictures\object1.pic"

Bullet1Tiles:
  .incbin "GamePictures\object2.pic"
