.segment "CODE"

;====================================================================================
; start_object_spawner
;====================================================================================

.proc start_object_spawner
  rep #$30    ; A=16, X/Y=16
  ldx #$0000

object_spawner_loop:
  lda a:ObjectTable,x
  beq object_spawner_loop_done
  txa
  clc
  adc #$0030
  tax
  cpx #ObjectTableSize
  bne object_spawner_loop

object_spawner_loop_done:
  lda a:NewObjectRequest
  sta a:ObjectTable,x
  rts

.endproc

;====================================================================================
; start_object_identifier
;====================================================================================

.proc start_object_identifier
  rep #$30    ; A=16, X/Y=16
  lda #ObjectTable
  sta a:ObjectOffset
  tcd

object_identifier_loop:
  lda Identity				;load the object identification byte of the object we're currently on
  beq next_object
  tax					;put the result in x so we can offset the object identification table to see what the object is
  jsr (ObjectIdentificationTable-2,x)	;jump to the code that corresponds with the object
  rep #$30    ; A=16, X/Y=16
next_object:
  lda a:ObjectOffset			;says how many objects have been identified
  clc
  adc #$0030				;add 48 to look at the next object
  sta a:ObjectOffset			;store the result for the next time we go through the loop
  tcd					;transfer the accumulator to y for "cpy #8192" and "lda ObjectTable,y"
  cmp #ObjectTableSize+ObjectTable	;sees if all objects have been identified (each object is 128 bytes)
  bne object_identifier_loop		;if so, quit searching
  lda #ObjectTable
  tcd
  jsr start_metasprite
  lda #$0000
  tcd
  jsr start_process_oam
  rts

.endproc

;====================================================================================
; ObjectIdentificationTable
;====================================================================================

ObjectIdentificationTable:
  .word object1,object2

;====================================================================================
; Objects
;====================================================================================
; Object 1
;====================================================================================

.proc object1
  rep #$30                  ; A=16, X/Y=16

up:
  lda a:Joy1Data
  and #$0800
  beq down

  lda YPosition
  sec
  sbc #$0002
  sta YPosition
  lda FlipBits
  ora #$8000
  sta FlipBits

down:
  lda a:Joy1Data
  and #$0400
  beq left

  lda YPosition
  clc
  adc #$0002
  sta YPosition
  lda FlipBits
  and #%0111111111111111
  sta FlipBits

left:
  lda a:Joy1Data
  and #$0200
  beq right

  dec a:MapX
  lda XPosition
  sec
  sbc #$0002
  sta XPosition
  lda FlipBits
  ora #$4000
  sta FlipBits

right:
  lda a:Joy1Data
  and #$0100
  beq done

  inc a:MapX
  lda XPosition
  clc
  adc #$0002
  sta XPosition
  lda FlipBits
  and #%1011111111111111
  sta FlipBits

done:
  rts

.endproc

;====================================================================================
;object2
;====================================================================================

.proc object2
  rts

.endproc
